package com.wily.jbley.supergnftests;

import java.util.HashMap;
import java.util.HashSet;

import junit.framework.TestCase;

import com.wily.introscope.agent.AgentShim;
import com.wily.introscope.agent.IAgent;
import com.wily.introscope.agent.trace.ITracerFactory;
import com.wily.introscope.agent.trace.InvocationData;
import com.wily.introscope.agent.trace.ProbeIdentification;
import com.wily.introscope.agent.trace.ProbeInformation;
import com.wily.introscope.spec.metric.AgentMetric;
import com.wily.introscope.spec.metric.MetricTypes;
import com.wily.introscope.stat.timeslice.IntegerTimeslicedValue;
import com.wily.jbley.supergnf.poll.Poll;
import com.wily.jbley.supergnf.poll.PollingService;
import com.wily.jbley.supergnf.poll.PollingTracer;
import com.wily.util.properties.AttributeListing;

public class PollingTests
    extends TestCase
{
    public static class LongTime
    {
        public int time()
        {
            try
            {
                Thread.sleep(5000);
            } catch (InterruptedException e)
            {
                // nop
            }
            return 4;
        }

        public String toString()
        {
            return "Long Time";
        }
    }

    protected IAgent agent;

    private void gc()
    {
        for (int i = 0; i < 3; i++)
        {
            System.gc();
            System.runFinalization();
            try
            {
                Thread.sleep(50);
            } catch (Exception e)
            {
                // nop
            }
        }
    }

    protected void setUp() throws Exception
    {
        this.agent = AgentShim.getAgent();
        this.agent.debug_setRetrievingValues(true);
    }

    protected void tearDown() throws Exception
    {
        this.agent.debug_setRetrievingValues(false);
        gc();
        Thread.sleep(1000);
    }

    public void testPollEquality()
    {
        Object o = new String("hello world");
        ProbeIdentification pi = new ProbeIdentification("java/lang/String",
                                                         "<init>",
                                                         "()V",
                                                         "java/lang/String");
        Poll p1 = new Poll(o, "{{this}.length()}", "TestMetrics:{{this}}", pi);
        Poll p2 = new Poll(o, "{{this}.length()}", "TestMetrics:{{this}}", pi);
        Poll p99 = new Poll(new Object(), "not really", "doesn't match", pi);
        assertEquals(p1.hashCode(), p1.hashCode());
        assertEquals(p1, p1);
        assertEquals(p2.hashCode(), p2.hashCode());
        assertEquals(p2, p2);
        assertEquals(p1.hashCode(), p2.hashCode());
        assertEquals(p1, p2);
        assertTrue(p1.hashCode() != p99.hashCode());
        assertTrue(!p1.equals(p99));
        assertTrue(!p2.equals(p99));

        HashSet set = new HashSet();
        set.add(p1);
        set.add(p2);
        set.add(p99);
        assertEquals(2, set.size());
        assertFalse(p1.equals("not the right type"));
    }

    public void testPollingService() throws Exception
    {
        String o = new String("hello world 2");
        AgentMetric metric = AgentMetric.getAgentMetric("TestMetrics:" + o,
                                                        MetricTypes.kIntegerFluctuatingCounter);
        ProbeIdentification pi = new ProbeIdentification("java/lang/String",
                                                         "<init>",
                                                         "()V",
                                                         "java/lang/String");
        Poll poll = new Poll(o,
                             "{{data.getInvocationObject().length()}}",
                             "TestMetrics:{{data.getInvocationObject()}}",
                             pi);
        PollingService.addPoll(poll);
        PollingService.addPoll(poll); // shouldn't get count of 2
        PollingService.doPolls(this.agent);

        IntegerTimeslicedValue value = (IntegerTimeslicedValue) this.agent.debug_retrieveCurrentValues(metric)[0];
        assertEquals(o.length(), value.getValue());
        assertEquals(1, value.getDataPointCount());

        // now test the GC behavior
        o = null;
        gc();
        PollingService.doPolls(this.agent);
        assertEquals(0, PollingService.polls.size());
    }

    // should really be last one since it starts a service
    public void testPollingServiceStartup() throws Exception
    {
        PollingService s = new PollingService();
        s.IAgentService_getServiceVersion();
        s.IAgentService_startService(this.agent, new HashMap());
        Thread.sleep(9000);
    }

    public void testPollingTakingTooLong() throws Exception
    {
        LongTime o = new LongTime();
        ProbeIdentification pi = new ProbeIdentification("java/lang/String",
                                                         "<init>",
                                                         "()V",
                                                         "java/lang/String");
        Poll poll = new Poll(o,
                             "{{data.getInvocationObject().time()}}",
                             "TestMetrics:{{data.getInvocationObject()}}",
                             pi);
        PollingService.addPoll(poll);
        PollingService.doPolls(this.agent);
    }

    public void testPollingTracer() throws Exception
    {
        String o = new String("hello world 3");
        ProbeIdentification pi = new ProbeIdentification("java/lang/String",
                                                         "<init>",
                                                         "()V",
                                                         "java/lang/String");

        AttributeListing props = new AttributeListing();
        props.put(PollingTracer.PollExpressionTracerParameterKey,
                  "{{data.getInvocationObject().length()}}");
        props.put(ITracerFactory.kNameKey,
                  "TestMetrics:{{data.getInvocationObject()}}");

        PollingTracer tracer = new PollingTracer(this.agent, props, pi, o);
        InvocationData data = InvocationData.debug_createInvocationData(this.agent,
                                                                        new ProbeInformation(this.agent,
                                                                                             pi,
                                                                                             new ITracerFactory[0]),
                                                                        o);
        tracer.ITracer_startTrace(0, data);
        tracer.ITracer_finishTrace(0, data);
        assertEquals(1, PollingService.polls.size());
    }

    public void testPollingTracerHasNoExpression() throws Exception
    {
        String o = new String("hello world 3");
        ProbeIdentification pi = new ProbeIdentification("java/lang/String",
                                                         "<init>",
                                                         "()V",
                                                         "java/lang/String");

        AttributeListing props = new AttributeListing();
        props.put(ITracerFactory.kNameKey,
                  "TestMetrics:{{data.getInvocationObject()}}");

        PollingTracer tracer = new PollingTracer(this.agent, props, pi, o);
        InvocationData data = InvocationData.debug_createInvocationData(this.agent,
                                                                        new ProbeInformation(this.agent,
                                                                                             pi,
                                                                                             new ITracerFactory[0]),
                                                                        o);
        tracer.ITracer_startTrace(0, data); // should cause warning message
        tracer.ITracer_finishTrace(0, data);
        tracer.ITracerFactory_getReentrancyLevel();
        tracer.ITracerFactory_isShutoff();
    }

    public void testPollInvoke() throws Exception
    {
        String o = new String("hello world");
        AgentMetric metric = AgentMetric.getAgentMetric("TestMetrics:" + o,
                                                        MetricTypes.kIntegerFluctuatingCounter);
        ProbeIdentification pi = new ProbeIdentification("java/lang/String",
                                                         "<init>",
                                                         "()V",
                                                         "java/lang/String");

        Poll poll = new Poll(o,
                             "{{data.getInvocationObject().length()}}",
                             "TestMetrics:{{data.getInvocationObject()}}",
                             pi);
        poll.invoke(this.agent);

        IntegerTimeslicedValue value = (IntegerTimeslicedValue) this.agent.debug_retrieveCurrentValues(metric)[0];
        assertEquals(o.length(), value.getValue());
    }
}
